==> medlogics-common-classes-1.1-20250515.180801-25.jar: com/floreantpos/model/dao/ChartOfAccountsDAO.java <==
package com.floreantpos.model.dao;

import java.io.Serializable;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;

import com.floreantpos.Messages;
import com.floreantpos.PosException;
import com.floreantpos.PosLog;
import com.floreantpos.model.ChartOfAccounts;
import com.floreantpos.swing.PaginationSupport;
import com.floreantpos.util.DefaultCOADataInserter;
import com.orocube.rest.service.server.BaseDataServiceDao;

public class ChartOfAccountsDAO extends BaseChartOfAccountsDAO {

	public ChartOfAccountsDAO() {
	}

	@Override
	protected void delete(Object obj, Session session) {
		ChartOfAccounts chartOfAccounts = (ChartOfAccounts) obj;
		if (chartOfAccounts == null) {
			throw new PosException("Chart of accounts is not found");
		}

		if (chartOfAccounts.isDefaultCoa()) {
			throw new PosException("Default chart of accounts cannot be deleted.");
		}
		chartOfAccounts.setDeleted(Boolean.TRUE);
		saveOrUpdate(chartOfAccounts, session);
	}

	@Override
	protected Serializable save(Object obj, Session s) {
		doCheckValidation(obj);
		updateTime(obj);
		return super.save(obj, s);
	}

	@Override
	protected void update(Object obj, Session s) {
		doCheckValidation(obj);
		updateTime(obj);
		super.update(obj, s);
	}

	@Override
	protected void saveOrUpdate(Object obj, Session s) {
		doCheckValidation(obj);
		updateTime(obj);
		super.saveOrUpdate(obj, s);
	}

	protected void doCheckValidation(Object obj) {
		ChartOfAccounts chartOfAccounts = (ChartOfAccounts) obj;

		String accountName = chartOfAccounts.getName();
		if (StringUtils.isNotBlank(accountName) && accountName.length() > 255) {
			throw new PosException(String.format(Messages.getString("value_too_long"), "account name", 255)); //$NON-NLS-1$
		}

		String accountCode = chartOfAccounts.getAccountCode();
		if (StringUtils.isNotBlank(accountCode) && accountCode.length() > 255) {
			throw new PosException(String.format(Messages.getString("value_too_long"), "account code", 255)); //$NON-NLS-1$
		}

		String description = chartOfAccounts.getDescription();
		if (StringUtils.isNotBlank(description) && description.length() > 512) {
			throw new PosException(String.format(Messages.getString("value_too_long"), "description", 512)); //$NON-NLS-1$
		}
	}

	@Override
	public List<ChartOfAccounts> findAll() {
		try (Session session = createNewSession()) {
			Criteria criteria = session.createCriteria(ChartOfAccounts.class);
			addDeletedFilter(criteria);
			criteria.addOrder(Order.asc(ChartOfAccounts.PROP_COA_ACCOUNT_TYPE_ID));
			criteria.addOrder(Order.asc(ChartOfAccounts.PROP_NAME));
			return criteria.list();
		}
	}

	public void findByAcountNameOrCode(PaginationSupport paginationSupport, String searchString) {
		Session session = null;
		try {
			session = createNewSession();
			Criteria criteria = session.createCriteria(ChartOfAccounts.class);
			addDeletedFilter(criteria);

			if (StringUtils.isNotEmpty(searchString)) {
				Criterion name = Restrictions.ilike(ChartOfAccounts.PROP_NAME, searchString.trim(), MatchMode.ANYWHERE);
				Criterion barcode = Restrictions.ilike(ChartOfAccounts.PROP_ACCOUNT_CODE, searchString.trim(), MatchMode.ANYWHERE);
				criteria.add(Restrictions.or(name, barcode));
			}
			criteria.setProjection(Projections.rowCount());

			Number rowCount = (Number) criteria.uniqueResult();
			if (rowCount != null) {
				paginationSupport.setNumRows(rowCount.intValue());
			}
			criteria.setProjection(null);
			criteria.addOrder(Order.asc(ChartOfAccounts.PROP_COA_ACCOUNT_TYPE_ID).ignoreCase());
			criteria.addOrder(Order.asc(ChartOfAccounts.PROP_NAME).ignoreCase());
			criteria.setFirstResult(paginationSupport.getCurrentRowIndex());
			criteria.setMaxResults(paginationSupport.getPageSize());
			paginationSupport.setRows(criteria.list());
		} finally {
			closeSession(session);
		}
	}

	public ChartOfAccounts findByAcountCode(String accountCode, Session session, boolean isInsertCOA) {
		if (StringUtils.isEmpty(accountCode)) {
			return null;
		}
		Criteria criteria = session.createCriteria(ChartOfAccounts.class);
		criteria.add(Restrictions.eq(ChartOfAccounts.PROP_ACCOUNT_CODE, accountCode.trim()));
		criteria.setMaxResults(1);
		ChartOfAccounts chartOfAccounts = (ChartOfAccounts) criteria.uniqueResult();
		if (isInsertCOA && chartOfAccounts == null) {
			chartOfAccounts = DefaultCOADataInserter.getChartOfAccountsByCode(accountCode);
		}
		return chartOfAccounts;
	}

	public void saveOrUpdateChartOfAccountsList(List<ChartOfAccounts> dataList, boolean updateLastUpdateTime, boolean updateSyncTime) throws Exception {
		if (dataList == null)
			return;

		Transaction tx = null;
		Session session = null;
		try {
			session = createNewSession();
			tx = session.beginTransaction();

			for (Iterator<ChartOfAccounts> iterator = dataList.iterator(); iterator.hasNext();) {
				ChartOfAccounts item = (ChartOfAccounts) iterator.next();
				ChartOfAccounts existingItem = get(item.getId());
				if (existingItem != null) {
					if (!BaseDataServiceDao.get().shouldSave(item.getLastUpdateTime(), existingItem.getLastUpdateTime())) {
						PosLog.info(getClass(), item.getName() + " already updated"); //$NON-NLS-1$
						continue;
					}
					PropertyUtils.copyProperties(existingItem, item);
					existingItem.setUpdateLastUpdateTime(updateLastUpdateTime);
					existingItem.setUpdateSyncTime(updateSyncTime);
					update(existingItem, session);
				}
				else {
					item.setUpdateLastUpdateTime(updateLastUpdateTime);
					item.setUpdateSyncTime(updateSyncTime);
					save(item, session);
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			throw e;
		} finally {
			closeSession(session);
		}
	}

}

==> medlogics-common-classes-1.1-20251204.120555-506.jar: com/floreantpos/model/dao/ChartOfAccountsDAO.java <==
package com.floreantpos.model.dao;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;

import com.floreantpos.Messages;
import com.floreantpos.PosException;
import com.floreantpos.PosLog;
import com.floreantpos.model.COAAccountType;
import com.floreantpos.model.ChartOfAccounts;
import com.floreantpos.model.util.DataProvider;
import com.floreantpos.swing.PaginationSupport;
import com.floreantpos.util.DefaultCOADataInserter;
import com.orocube.rest.service.server.BaseDataServiceDao;

public class ChartOfAccountsDAO extends BaseChartOfAccountsDAO {

	public ChartOfAccountsDAO() {
	}

	@Override
	protected void delete(Object obj, Session session) {
		ChartOfAccounts chartOfAccounts = (ChartOfAccounts) obj;
		if (chartOfAccounts == null) {
			throw new PosException("Chart of accounts is not found");
		}

		if (chartOfAccounts.isDefaultCoa()) {
			throw new PosException("Default chart of accounts cannot be deleted.");
		}
		chartOfAccounts.setDeleted(Boolean.TRUE);
		saveOrUpdate(chartOfAccounts, session);
	}

	@Override
	protected Serializable save(Object obj, Session s) {
		doCheckValidation(obj);
		updateTime(obj);
		return super.save(obj, s);
	}

	@Override
	protected void update(Object obj, Session s) {
		doCheckValidation(obj);
		updateTime(obj);
		super.update(obj, s);
	}

	@Override
	protected void saveOrUpdate(Object obj, Session s) {
		doCheckValidation(obj);
		updateTime(obj);
		super.saveOrUpdate(obj, s);
	}

	protected void doCheckValidation(Object obj) {
		ChartOfAccounts chartOfAccounts = (ChartOfAccounts) obj;

		String accountName = chartOfAccounts.getName();
		if (StringUtils.isNotBlank(accountName) && accountName.length() > 255) {
			throw new PosException(String.format(Messages.getString("value_too_long"), "account name", 255)); //$NON-NLS-1$
		}

		String accountCode = chartOfAccounts.getAccountCode();
		if (StringUtils.isNotBlank(accountCode) && accountCode.length() > 255) {
			throw new PosException(String.format(Messages.getString("value_too_long"), "account code", 255)); //$NON-NLS-1$
		}

		String description = chartOfAccounts.getDescription();
		if (StringUtils.isNotBlank(description) && description.length() > 512) {
			throw new PosException(String.format(Messages.getString("value_too_long"), "description", 512)); //$NON-NLS-1$
		}
	}

	@Override
	public List<ChartOfAccounts> findAll() {
		try (Session session = createNewSession()) {
			Criteria criteria = session.createCriteria(ChartOfAccounts.class);
			addDeletedFilter(criteria);
			criteria.addOrder(Order.asc(ChartOfAccounts.PROP_COA_ACCOUNT_TYPE_ID));
			criteria.addOrder(Order.asc(ChartOfAccounts.PROP_NAME));
			return criteria.list();
		}
	}

	public void findByAcountNameOrCode(PaginationSupport paginationSupport, String searchString) {
		findByAcountNameOrCode(paginationSupport, searchString, null);
	}

	public void findByAcountNameOrCode(PaginationSupport paginationSupport, String searchString, COAAccountType coaAccountType) {
		Session session = null;
		try {
			session = createNewSession();
			Criteria criteria = session.createCriteria(ChartOfAccounts.class);
			addDeletedFilter(criteria);

			if (StringUtils.isNotEmpty(searchString)) {
				Criterion name = Restrictions.ilike(ChartOfAccounts.PROP_NAME, searchString.trim(), MatchMode.ANYWHERE);
				Criterion barcode = Restrictions.ilike(ChartOfAccounts.PROP_ACCOUNT_CODE, searchString.trim(), MatchMode.ANYWHERE);
				criteria.add(Restrictions.or(name, barcode));
			}

			if (coaAccountType != null) {
				criteria.add(Restrictions.eq(ChartOfAccounts.PROP_COA_ACCOUNT_TYPE_ID, coaAccountType.getId()));
			}

			criteria.setProjection(Projections.rowCount());

			Number rowCount = (Number) criteria.uniqueResult();
			if (rowCount != null) {
				paginationSupport.setNumRows(rowCount.intValue());
			}
			criteria.setProjection(null);
			criteria.addOrder(Order.asc(ChartOfAccounts.PROP_COA_ACCOUNT_TYPE_ID).ignoreCase());
			criteria.addOrder(Order.asc(ChartOfAccounts.PROP_NAME).ignoreCase());
			criteria.setFirstResult(paginationSupport.getCurrentRowIndex());
			criteria.setMaxResults(paginationSupport.getPageSize());
			paginationSupport.setRows(criteria.list());
		} finally {
			closeSession(session);
		}
	}

	public ChartOfAccounts findByAcountCode(String accountCode, Session session, boolean isInsertCOA) {
		if (StringUtils.isEmpty(accountCode)) {
			return null;
		}
		Criteria criteria = session.createCriteria(ChartOfAccounts.class);
		criteria.add(Restrictions.eq(ChartOfAccounts.PROP_ACCOUNT_CODE, accountCode.trim()));
		criteria.setMaxResults(1);
		ChartOfAccounts chartOfAccounts = (ChartOfAccounts) criteria.uniqueResult();
		if (isInsertCOA && chartOfAccounts == null) {
			chartOfAccounts = DefaultCOADataInserter.getChartOfAccountsByCode(accountCode);
		}
		return chartOfAccounts;
	}

	public List<ChartOfAccounts> getAccountsByType(String coaTypeID) {
		return getAccountsByType(Arrays.asList(coaTypeID));
	}

	public List<ChartOfAccounts> getAccountsByType(List<String> coaTypeIDs) {
		try (Session session = createNewSession()) {
			Criteria criteria = session.createCriteria(ChartOfAccounts.class);
			addDeletedFilter(criteria);

			if (!coaTypeIDs.isEmpty()) {
				criteria.add(Restrictions.in(ChartOfAccounts.PROP_COA_ACCOUNT_TYPE_ID, coaTypeIDs));
			}

			criteria.addOrder(Order.asc(ChartOfAccounts.PROP_COA_ACCOUNT_TYPE_ID));
			criteria.addOrder(Order.asc(ChartOfAccounts.PROP_NAME));
			return criteria.list();
		}
	}

	public List<String> getCOACodsByTypeIds(List<String> coaTypeIDs) {
		try (Session session = createNewSession()) {
			Criteria criteria = session.createCriteria(ChartOfAccounts.class);
			addDeletedFilter(criteria);

			if (!coaTypeIDs.isEmpty()) {
				criteria.add(Restrictions.in(ChartOfAccounts.PROP_COA_ACCOUNT_TYPE_ID, coaTypeIDs));
			}

			criteria.setProjection(Projections.property(ChartOfAccounts.PROP_ACCOUNT_CODE));
			return criteria.list();
		}
	}

	public List<ChartOfAccounts> getCOAs(Collection<String> codeList) {
		List list = new ArrayList<ChartOfAccounts>();
		if (codeList == null) {
			return list;
		}
		try (Session session = createNewSession()) {
			for (String code : codeList) {
				ChartOfAccounts coaFromMap = DataProvider.get().getCOAFromMap(code, session, true);
				list.add(coaFromMap);
			}
			return list;
		}
	}

	public void saveOrUpdateChartOfAccountsList(List<ChartOfAccounts> dataList, boolean updateLastUpdateTime, boolean updateSyncTime) throws Exception {
		if (dataList == null)
			return;

		Transaction tx = null;
		Session session = null;
		try {
			session = createNewSession();
			tx = session.beginTransaction();

			for (Iterator<ChartOfAccounts> iterator = dataList.iterator(); iterator.hasNext();) {
				ChartOfAccounts item = (ChartOfAccounts) iterator.next();
				ChartOfAccounts existingItem = get(item.getId());
				if (existingItem != null) {
					if (!BaseDataServiceDao.get().shouldSave(item.getLastUpdateTime(), existingItem.getLastUpdateTime())) {
						PosLog.info(getClass(), item.getName() + " already updated"); //$NON-NLS-1$
						continue;
					}
					PropertyUtils.copyProperties(existingItem, item);
					existingItem.setUpdateLastUpdateTime(updateLastUpdateTime);
					existingItem.setUpdateSyncTime(updateSyncTime);
					update(existingItem, session);
				}
				else {
					item.setUpdateLastUpdateTime(updateLastUpdateTime);
					item.setUpdateSyncTime(updateSyncTime);
					save(item, session);
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			throw e;
		} finally {
			closeSession(session);
		}
	}

	//	public boolean hasChartOfAccountsData(Session session) {
	//		//		try (Session session = createNewSession()) {
	//		Criteria criteria = session.createCriteria(ChartOfAccounts.class);
	//		criteria.setProjection(Projections.rowCount());
	//		Number result = (Number) criteria.uniqueResult();
	//		if (result != null) {
	//			return result.intValue() > 0;
	//		}
	//		return false;
	//	}
	//	}

}
